# -*- coding: utf-8 -*-

# Copyright (C) 2007 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Base class for single-instance classes."""

__all__ = ("Singleton",)


class Singleton:

    """Base class for single-instance classes."""

    def __new__(cls, *args, **kwargs):
        """Return possibly existing instance."""
        if not "_instance" in cls.__dict__:
            cls._instance = object.__new__(cls, *args, **kwargs)
        return cls._instance
